
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/time.h>
#include <netdb.h>
#include <string.h>
#include <unistd.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <errno.h>
#include <fcntl.h>
#ifndef MSG_CONFIRM
#define MSG_CONFIRM 0
#endif
#ifndef MSG_NOSIGNAL
#define MSG_NOSIGNAL 0
#endif
#ifndef MSG_DONTWAIT
#define MSG_DONTWAIT 0
#endif
#ifndef MSG_EOR
#define MSG_EOR 0
#endif

static void *
safe_buffer_pointer(cl_object x, cl_index size)
{
	cl_type t = type_of(x);
	int ok = 0;
	if (t == t_base_string) {
		ok = (size <= x->base_string.dim);
	} else if (t == t_vector) {
		cl_elttype aet = (cl_elttype)x->vector.elttype;
		if (aet == aet_b8 || aet == aet_i8 || aet == aet_bc) {
			ok = (size <= x->vector.dim);
		} else if (aet == aet_fix || aet == aet_index) {
			cl_index divisor = sizeof(cl_index);
			size = (size + divisor - 1) / divisor;
			ok = (size <= x->vector.dim);
		}
	}
	if (!ok) {
		FEerror("Lisp object is not a valid socket buffer: ~A", 1, x);
	}
	return (void *)x->vector.self.t;
}


static void fill_inet_sockaddr(struct sockaddr_in *sockaddr, int port,
			       int a1, int a2, int a3, int a4)
{
#if defined(_MSC_VER) || defined(mingw32)
	memset(sockaddr,0,sizeof(struct sockaddr_in));
#else
	bzero(sockaddr,sizeof(struct sockaddr_in));
#endif
	sockaddr->sin_family = AF_INET;
	sockaddr->sin_port = htons(port);
	sockaddr->sin_addr.s_addr= htonl((uint32_t)a1<<24 | (uint32_t)a2<<16 | (uint32_t)a3<<8 | (uint32_t)a4) ;

}

#ifdef __cplusplus
extern "C" {
#endif
static cl_object L1ff_socket(cl_object , cl_object , cl_object );
#define VT2
#define VLEX2
#define CLSR2
#define STCK2
static cl_object L2ff_listen(cl_object , cl_object );
#define VT3
#define VLEX3
#define CLSR3
#define STCK3
static cl_object L3ff_close(cl_object );
#define VT4
#define VLEX4
#define CLSR4
#define STCK4
static cl_object L5split(cl_narg, cl_object , ...);
#define VT5 cl_object T0,T1;
#define VLEX5
#define CLSR5
#define STCK5 cl_object env0; cl_object CLV0;
static cl_object LC4is_ws(cl_narg, cl_object , ...);
#define VT6
#define VLEX6
#define CLSR6
#define STCK6 cl_object CLV0;
#ifdef __cplusplus
extern cl_object clos_ensure_class(...);
#else
extern cl_object clos_ensure_class();
#endif
#ifdef __cplusplus
extern cl_object clos_install_method(...);
#else
extern cl_object clos_install_method();
#endif
static cl_object LC6__g20(cl_object );
#define VT7 cl_object T0;
#define VLEX7
#define CLSR7
#define STCK7
static cl_object L7get_host_by_name(cl_object );
#define VT8 cl_object T0,T1,T2,T3;
#define VLEX8
#define CLSR8
#define STCK8
static cl_object L10get_host_by_address(cl_object volatile );
#define VT9 cl_object T0,T1,T2,T3;
#define VLEX9
#define CLSR9
#define STCK9 cl_object volatile env0; cl_object volatile CLV0,CLV1;
static cl_object LC8__g28(cl_narg, ...);
#define VT10
#define VLEX10
#define CLSR10
#define STCK10 cl_object CLV0,CLV1;
static cl_object LC9__g29(cl_object );
#define VT11
#define VLEX11
#define CLSR11
#define STCK11
static cl_object LC11__g35();
#define VT12
#define VLEX12
#define CLSR12
#define STCK12
static cl_object LC13__g36(cl_object , cl_object );
#define VT13
#define VLEX13
#define CLSR13
#define STCK13 cl_object env0; cl_object CLV0,CLV1;
static cl_object LC12si___print_unreadable_object_body_(cl_narg, ...);
#ifdef __cplusplus
extern cl_object cl_slot_value(...);
#else
extern cl_object cl_slot_value();
#endif
#define VT14 cl_object T0;
#define VLEX14
#define CLSR14
#define STCK14 cl_object CLV0,CLV1;
static cl_object LC14__g51(cl_narg, cl_object , cl_object , ...);
#define LC14__g51keys (&VV[181])
#ifdef __cplusplus
extern cl_object cl_slot_boundp(...);
#else
extern cl_object cl_slot_boundp();
#endif
#ifdef __cplusplus
extern cl_object si_ecase_error(...);
#else
extern cl_object si_ecase_error();
#endif
#define VT15 cl_object T0,T1;
#define VLEX15
#define CLSR15
#define STCK15
static cl_object LC15__g63(cl_object , cl_object );
#define VT16 cl_object T0;
#define VLEX16
#define CLSR16
#define STCK16
static cl_object LC16__g64(cl_object );
#define VT17 cl_object T0;
#define VLEX17
#define CLSR17
#define STCK17
static cl_object LC17__g71(cl_object );
#ifdef __cplusplus
extern cl_object cl_slot_makunbound(...);
#else
extern cl_object cl_slot_makunbound();
#endif
#define VT18 cl_object T0;
#define VLEX18
#define CLSR18
#define STCK18
static cl_object LC18__g75(cl_narg, cl_object , cl_object , cl_object , ...);
#define LC18__g75keys (&VV[187])
#define VT19 cl_object T0;
#define VLEX19
#define CLSR19
#define STCK19
static cl_object L19get_protocol_by_name(cl_object );
#define VT20
#define VLEX20
#define CLSR20
#define STCK20
static cl_object L20make_inet_address(cl_object );
#define VT21 cl_object T0,T1;
#define VLEX21
#define CLSR21
#define STCK21
static cl_object LC21__g79();
#define VT22
#define VLEX22
#define CLSR22
#define STCK22
static cl_object L22make_inet_socket(cl_object , cl_object );
#define VT23
#define VLEX23
#define CLSR23
#define STCK23
static cl_object LC25__g92(volatile cl_narg, cl_object volatile , ...);
#define VT24 cl_object T0,T1,T2,T3,T4,T5;
#define VLEX24
#define CLSR24
#define STCK24 cl_object volatile env0; cl_object volatile CLV0,CLV1;
static cl_object LC23__g98(cl_narg, ...);
#define VT25
#define VLEX25
#define CLSR25
#define STCK25 cl_object CLV0,CLV1;
static cl_object LC24__g99(cl_object );
#define VT26
#define VLEX26
#define CLSR26
#define STCK26
static cl_object LC26__g103(cl_object );
#define VT27 cl_object T0,T1,T2,T3;
#define VLEX27
#define CLSR27
#define STCK27
static cl_object LC27__g104(cl_narg, cl_object , ...);
#define VT28 cl_object T0,T1,T2,T3,T4,T5;
#define VLEX28
#define CLSR28
#define STCK28
static cl_object LC28__g105(cl_object );
#define VT29
#define VLEX29
#define CLSR29
#define STCK29
static cl_object LC29__g106(cl_object );
#define VT30
#define VLEX30
#define CLSR30
#define STCK30
static cl_object LC34__g127(volatile cl_narg, cl_object volatile , cl_object volatile , cl_object volatile , ...);
#define LC34__g127keys (&VV[196])
#define VT31 cl_object T0,T1,T2,T3,T4,T5,T6,T7,T8;
#define VLEX31
#define CLSR31
#define STCK31 cl_object volatile env0; cl_object volatile CLV0,CLV1;
static cl_object LC30__g133(cl_narg, ...);
#define VT32
#define VLEX32
#define CLSR32
#define STCK32 cl_object CLV0,CLV1;
static cl_object LC31__g134(cl_object );
#define VT33
#define VLEX33
#define CLSR33
#define STCK33
static cl_object LC32__g142(cl_narg, ...);
#define VT34
#define VLEX34
#define CLSR34
#define STCK34 cl_object CLV0,CLV1;
static cl_object LC33__g143(cl_object );
#define VT35
#define VLEX35
#define CLSR35
#define STCK35
static cl_object LC35__g145();
#define VT36
#define VLEX36
#define CLSR36
#define STCK36
static cl_object LC38__g158(volatile cl_narg, cl_object volatile , ...);
#define VT37 cl_object T0,T1,T2,T3;
#define VLEX37
#define CLSR37
#define STCK37 cl_object volatile env0; cl_object volatile CLV0,CLV1;
static cl_object LC39__g170(cl_object );
#define VT38 cl_object T0,T1,T2,T3;
#define VLEX38
#define CLSR38
#define STCK38
static cl_object LC42__g183(volatile cl_narg, cl_object volatile , ...);
#define VT39 cl_object T0,T1,T2,T3;
#define VLEX39
#define CLSR39
#define STCK39 cl_object volatile env0; cl_object volatile CLV0,CLV1;
static cl_object LC43__g195(cl_object );
#define VT40
#define VLEX40
#define CLSR40
#define STCK40
static cl_object LC36__g164(cl_narg, ...);
#define VT41
#define VLEX41
#define CLSR41
#define STCK41 cl_object CLV0,CLV1;
static cl_object LC37__g165(cl_object );
#define VT42
#define VLEX42
#define CLSR42
#define STCK42
static cl_object LC40__g189(cl_narg, ...);
#define VT43
#define VLEX43
#define CLSR43
#define STCK43 cl_object CLV0,CLV1;
static cl_object LC41__g190(cl_object );
#define VT44
#define VLEX44
#define CLSR44
#define STCK44
static cl_object LC44__g196(cl_object );
#define VT45 cl_object T0,T1;
#define VLEX45
#define CLSR45
#define STCK45
static cl_object LC45__g197(cl_object , cl_object );
#define VT46 cl_object T0;
#define VLEX46
#define CLSR46
#define STCK46
static cl_object L48make_stream_from_fd(volatile cl_narg, cl_object volatile , cl_object volatile , cl_object volatile , ...);
#define VT47 cl_object T0,T1,T2,T3;
#define VLEX47
#define CLSR47
#define STCK47 cl_object volatile env0; cl_object volatile CLV0,CLV1;
static cl_object LC46__g204(cl_narg, ...);
#define VT48
#define VLEX48
#define CLSR48
#define STCK48 cl_object CLV0,CLV1;
static cl_object LC47__g205(cl_object );
#define VT49
#define VLEX49
#define CLSR49
#define STCK49
static cl_object LC49__g220(cl_narg, cl_object , ...);
#define LC49__g220keys (&VV[206])
#define VT50
#define VLEX50
#define CLSR50
#define STCK50
static cl_object LC50__g225(cl_object , cl_object );
#define VT51 cl_object T0,T1,T2;
#define VLEX51
#define CLSR51
#define STCK51
static cl_object LC51define_socket_condition(cl_object , cl_object );
#ifdef __cplusplus
extern cl_object si_dm_too_few_arguments(...);
#else
extern cl_object si_dm_too_few_arguments();
#endif
#ifdef __cplusplus
extern cl_object si_check_arg_length(...);
#else
extern cl_object si_check_arg_length();
#endif
#define VT52 cl_object T0,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11;
#define VLEX52
#define CLSR52
#define STCK52
static cl_object L52condition_for_errno(cl_object );
#define VT53 cl_object T0;
#define VLEX53
#define CLSR53
#define STCK53
static cl_object L53socket_error(cl_object );
#define VT54
#define VLEX54
#define CLSR54
#define STCK54
static cl_object L54name_service_error(cl_object );
#define VT55 cl_object T0;
#define VLEX55
#define CLSR55
#define STCK55
static cl_object LC55__g250(cl_object , cl_object );
#define VT56 cl_object T0,T1,T2;
#define VLEX56
#define CLSR56
#define STCK56
static cl_object LC56define_name_service_condition(cl_object , cl_object );
#define VT57 cl_object T0,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11;
#define VLEX57
#define CLSR57
#define STCK57
static cl_object L57condition_for_name_service_errno(cl_object );
#define VT58 cl_object T0;
#define VLEX58
#define CLSR58
#define STCK58
static cl_object L58get_name_service_errno();
#define VT59
#define VLEX59
#define CLSR59
#define STCK59
static cl_object L59get_name_service_error_message(cl_object );
#define VT60
#define VLEX60
#define CLSR60
#define STCK60
static cl_object L60get_sockopt_int(cl_object , cl_object );
#define VT61 cl_object T0;
#define VLEX61
#define CLSR61
#define STCK61
static cl_object L61get_sockopt_bool(cl_object , cl_object );
#define VT62 cl_object T0;
#define VLEX62
#define CLSR62
#define STCK62
static cl_object L62get_sockopt_timeval(cl_object , cl_object );
#define VT63 cl_object T0;
#define VLEX63
#define CLSR63
#define STCK63
static cl_object L63set_sockopt_int(cl_object , cl_object , cl_object );
#define VT64 cl_object T0;
#define VLEX64
#define CLSR64
#define STCK64
static cl_object L64set_sockopt_bool(cl_object , cl_object , cl_object );
#define VT65 cl_object T0;
#define VLEX65
#define CLSR65
#define STCK65
static cl_object L65set_sockopt_timeval(cl_object , cl_object , cl_object );
#define VT66 cl_object T0;
#define VLEX66
#define CLSR66
#define STCK66
static cl_object LC66define_sockopt(cl_object , cl_object );
#define VT67 cl_object T0,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13;
#define VLEX67
#define CLSR67
#define STCK67
static cl_object L67sockopt_type(cl_object );
#define VT68 cl_object T0,T1;
#define VLEX68
#define CLSR68
#define STCK68
static cl_object L68sockopt_receive_buffer(cl_object );
#define VT69 cl_object T0,T1;
#define VLEX69
#define CLSR69
#define STCK69
static cl_object L69_setf_sockopt_receive_buffer_(cl_object , cl_object );
#define VT70 cl_object T0,T1;
#define VLEX70
#define CLSR70
#define STCK70
static cl_object L70sockopt_receive_timeout(cl_object );
#define VT71 cl_object T0,T1;
#define VLEX71
#define CLSR71
#define STCK71
static cl_object L71_setf_sockopt_receive_timeout_(cl_object , cl_object );
#define VT72 cl_object T0,T1;
#define VLEX72
#define CLSR72
#define STCK72
static cl_object L72sockopt_send_timeout(cl_object );
#define VT73 cl_object T0,T1;
#define VLEX73
#define CLSR73
#define STCK73
static cl_object L73_setf_sockopt_send_timeout_(cl_object , cl_object );
#define VT74 cl_object T0,T1;
#define VLEX74
#define CLSR74
#define STCK74
static cl_object L74sockopt_reuse_address(cl_object );
#define VT75 cl_object T0,T1;
#define VLEX75
#define CLSR75
#define STCK75
static cl_object L75_setf_sockopt_reuse_address_(cl_object , cl_object );
#define VT76 cl_object T0,T1;
#define VLEX76
#define CLSR76
#define STCK76
static cl_object L76sockopt_keep_alive(cl_object );
#define VT77 cl_object T0,T1;
#define VLEX77
#define CLSR77
#define STCK77
static cl_object L77_setf_sockopt_keep_alive_(cl_object , cl_object );
#define VT78 cl_object T0,T1;
#define VLEX78
#define CLSR78
#define STCK78
static cl_object L78socket_dont_route(cl_object );
#define VT79 cl_object T0,T1;
#define VLEX79
#define CLSR79
#define STCK79
static cl_object L79_setf_socket_dont_route_(cl_object , cl_object );
#define VT80 cl_object T0,T1;
#define VLEX80
#define CLSR80
#define STCK80
static cl_object L80socket_linger(cl_object );
#define VT81 cl_object T0,T1;
#define VLEX81
#define CLSR81
#define STCK81
static cl_object L81_setf_socket_linger_(cl_object , cl_object );
#define VT82 cl_object T0,T1;
#define VLEX82
#define CLSR82
#define STCK82
static cl_object L82sockopt_tcp_nodelay(cl_object );
#define VT83 cl_object T0,T1;
#define VLEX83
#define CLSR83
#define STCK83
static cl_object L83_setf_sockopt_tcp_nodelay_(cl_object , cl_object );
#define VT84 cl_object T0,T1;
#define VLEX84
#define CLSR84
#define STCK84
#define VT1 cl_object T0,T1,T2,T3,T4;
#define VLEX1
#define CLSR1
#define STCK1
static cl_object Cblock;
#define VM1 5
#define VM84 2
#define VM83 2
#define VM82 2
#define VM81 2
#define VM80 2
#define VM79 2
#define VM78 2
#define VM77 2
#define VM76 2
#define VM75 2
#define VM74 2
#define VM73 2
#define VM72 2
#define VM71 2
#define VM70 2
#define VM69 2
#define VM68 2
#define VM67 14
#define VM66 1
#define VM65 1
#define VM64 1
#define VM63 1
#define VM62 1
#define VM61 1
#define VM60 0
#define VM59 0
#define VM58 1
#define VM57 12
#define VM56 3
#define VM55 1
#define VM54 0
#define VM53 1
#define VM52 12
#define VM51 3
#define VM50 0
#define VM49 0
#define VM48 0
#define VM47 4
#define VM46 1
#define VM45 2
#define VM44 0
#define VM43 0
#define VM42 0
#define VM41 0
#define VM40 0
#define VM39 4
#define VM38 4
#define VM37 4
#define VM36 0
#define VM35 0
#define VM34 0
#define VM33 0
#define VM32 0
#define VM31 9
#define VM30 0
#define VM29 0
#define VM28 6
#define VM27 4
#define VM26 0
#define VM25 0
#define VM24 6
#define VM23 0
#define VM22 0
#define VM21 2
#define VM20 0
#define VM19 1
#define VM18 1
#define VM17 1
#define VM16 1
#define VM15 2
#define VM14 1
#define VM13 0
#define VM12 0
#define VM11 0
#define VM10 0
#define VM9 4
#define VM8 4
#define VM7 1
#define VM6 0
#define VM5 2
#define VM4 0
#define VM3 0
#define VM2 0
#define VM 243
#define VMtemp 43
#ifdef ECL_DYNAMIC_VV
static cl_object *VV;
#else
static cl_object VV[VM];
#endif
#ifdef __cplusplus
}
#endif
/*
 * Statically defined constants
 */
ecl_def_ct_base_string(_ecl_static_46,"SET-SOCKOPT-~A",14,static,const);
ecl_def_ct_base_string(_ecl_static_45,"GET-SOCKOPT-~A",14,static,const);
ecl_def_ct_base_string(_ecl_static_44,"Sockopt error: ~A",17,static,const);
ecl_def_ct_base_string(_ecl_static_43,"Name service error in \"~A\": ~A (~A)",35,static,const);
ecl_def_ct_base_string(_ecl_static_42,"an unknown location",19,static,const);
ecl_def_ct_base_string(_ecl_static_41,"Common base class of socket related conditions.",47,static,const);
ecl_def_ct_base_string(_ecl_static_40,"Socket error in \"~A\": ~A (~A)",29,static,const);
ecl_def_ct_base_string(_ecl_static_39,"No next method.",15,static,const);
ecl_def_ct_base_string(_ecl_static_38,"outer space",11,static,const);
ecl_def_ct_base_string(_ecl_static_37,"name must be a string.",22,static,const);
ecl_def_ct_base_string(_ecl_static_36,"FD-STREAM",9,static,const);
ecl_def_ct_base_string(_ecl_static_35,"fcntl",5,static,const);
ecl_def_ct_base_string(_ecl_static_34,"Socket-connect needs two parameters for local sockets.",54,static,const);
ecl_def_ct_base_string(_ecl_static_33,"Socket-bind needs two parameters for local sockets.",51,static,const);
ecl_def_ct_base_string(_ecl_static_32,"Class representing local domain (AF_LOCAL) sockets,\nalso known as unix-domain so"
 "ckets.",86,static,const);
ecl_def_ct_base_string(_ecl_static_31,"send",4,static,const);
ecl_def_ct_base_string(_ecl_static_30,"getsockname",11,static,const);
ecl_def_ct_base_string(_ecl_static_29,"getpeername",11,static,const);
ecl_def_ct_base_string(_ecl_static_28,"connect",7,static,const);
ecl_def_ct_base_string(_ecl_static_27,"accept",6,static,const);
ecl_def_ct_base_string(_ecl_static_26,"bind",4,static,const);
ecl_def_ct_base_string(_ecl_static_25,"Socket-bind needs three parameters for inet sockets.",52,static,const);
ecl_def_ct_base_string(_ecl_static_24,"Class representing TCP and UDP sockets.\n\nExamples:\n\n (make-instance 'inet-socket"
 " :type :stream :protocol :tcp)\n\n (make-instance 'inet-socket :type :datagram :pr"
 "otocol :udp)\n",173,static,const);
ecl_def_ct_base_string(_ecl_static_23,"receive",7,static,const);
ecl_def_ct_base_string(_ecl_static_22,"You have to supply either buffer or length!",43,static,const);
ecl_def_ct_base_string(_ecl_static_21,"close",5,static,const);
ecl_def_ct_base_string(_ecl_static_20,"listen",6,static,const);
ecl_def_ct_base_string(_ecl_static_19,"Close SOCKET at low level. Do not use directly.",47,static,const);
ecl_def_ct_base_string(_ecl_static_18,"Put SOCKET in non-blocking mode - or not, according to NON-BLOCKING-P",69,static,const);
ecl_def_ct_base_string(_ecl_static_17,"Is SOCKET in non-blocking mode?",31,static,const);
ecl_def_ct_base_string(_ecl_static_16,"Find or create a STREAM that can be used for IO\non SOCKET (which must be connect"
 "ed).  ARGS are passed onto\nSB-SYS:MAKE-FD-STREAM.",129,static,const);
ecl_def_ct_base_string(_ecl_static_15,"Close SOCKET.  May throw any kind of error that write(2) would have\nthrown.  If "
 "SOCKET-MAKE-STREAM has been called, calls CLOSE on that\nstream instead",150,static,const);
ecl_def_ct_base_string(_ecl_static_14,"Send length octets from buffer into socket, using sendto(2).\nIf buffer is a stri"
 "ng, it will converted to octets according to external-format&\nIf length is nil, "
 "the length of the octet buffer is used. The format of address\ndepends on the soc"
 "ket type (for example for inet domain sockets it would be a \nlist of an ip addre"
 "ss and a port). If no socket address is provided, send(2) \nwill be called instea"
 "d. Returns the number of octets written.",440,static,const);
ecl_def_ct_base_string(_ecl_static_13,"Read LENGTH octets from SOCKET into BUFFER (or a freshly-consed buffer if\nNIL), "
 "using recvfrom(2).  If LENGTH is NIL, the length of BUFFER is\nused, so at least "
 "one of these two arguments must be non-NIL.  If\nBUFFER is supplied, it had bette"
 "r be of an element type one octet wide.\nReturns the buffer, its length, and the "
 "address of the peer\nthat sent it, as multiple values.  On datagram sockets, sets"
 " MSG_TRUNC\nso that the actual packet length is returned even if the buffer was t"
 "oo\nsmall",488,static,const);
ecl_def_ct_base_string(_ecl_static_12,"Mark SOCKET as willing to accept incoming connections.  BACKLOG\ndefines the maxi"
 "mum length that the queue of pending connections may\ngrow to before new connecti"
 "on attempts are refused.  See also listen(2)",204,static,const);
ecl_def_ct_base_string(_ecl_static_11,"Return the address (as vector of bytes) and port\n  that the socket is bound to, "
 "as multiple values.",99,static,const);
ecl_def_ct_base_string(_ecl_static_10,"Return the socket's peer; depending on the address\n  family this may return mult"
 "iple values",91,static,const);
ecl_def_ct_base_string(_ecl_static_9,"Perform the connect(2) call to connect SOCKET to a\n  remote PEER.  No useful ret"
 "urn value.",90,static,const);
ecl_def_ct_base_string(_ecl_static_8,"Perform the accept(2) call, returning a\nnewly-created connected socket and the p"
 "eer address as multiple\nvalues",110,static,const);
ecl_def_ct_base_string(_ecl_static_7,"Bind SOCKET to ADDRESS, which may vary according to\nsocket family.  For the INET"
 " family, pass ADDRESS and PORT as two\narguments; for FILE address family sockets"
 ", pass the filename string.\nSee also bind(2)",204,static,const);
ecl_def_ct_base_string(_ecl_static_6,"socket",6,static,const);
ecl_def_ct_base_string(_ecl_static_5,"Common base class of all sockets, not ment to be\ndirectly instantiated.",71,static,const);
ecl_def_ct_base_string(_ecl_static_4,"No socket family",16,static,const);
ecl_def_ct_base_string(_ecl_static_3,"get-host-by-address",19,static,const);
ecl_def_ct_base_string(_ecl_static_2,"get-host-by-name",16,static,const);
ecl_def_ct_base_string(_ecl_static_1,"",0,static,const);
ecl_def_ct_base_string(_ecl_static_0,"SB-BSD-SOCKETS",14,static,const);
/*
 * Exported Lisp functions
 */
#define compiler_cfuns_size 42
static const struct ecl_cfun compiler_cfuns[] = {
 /*t,m,narg,padding,name,block,entry*/
{0,0,3,0,MAKE_FIXNUM(164),MAKE_FIXNUM(4),(cl_objectfn)L1ff_socket,Cnil,MAKE_FIXNUM(2653)},
{0,0,2,0,MAKE_FIXNUM(165),MAKE_FIXNUM(5),(cl_objectfn)L2ff_listen,Cnil,MAKE_FIXNUM(2724)},
{0,0,1,0,MAKE_FIXNUM(166),MAKE_FIXNUM(6),(cl_objectfn)L3ff_close,Cnil,MAKE_FIXNUM(2790)},
{0,0,-1,0,MAKE_FIXNUM(167),MAKE_FIXNUM(8),(cl_objectfn)L5split,Cnil,MAKE_FIXNUM(3073)},
{0,0,1,0,MAKE_FIXNUM(169),MAKE_FIXNUM(12),(cl_objectfn)L7get_host_by_name,Cnil,MAKE_FIXNUM(4564)},
{0,0,1,0,MAKE_FIXNUM(174),MAKE_FIXNUM(18),(cl_objectfn)L10get_host_by_address,Cnil,MAKE_FIXNUM(6336)},
{0,0,1,0,MAKE_FIXNUM(191),MAKE_FIXNUM(43),(cl_objectfn)L19get_protocol_by_name,Cnil,MAKE_FIXNUM(17161)},
{0,0,1,0,MAKE_FIXNUM(192),MAKE_FIXNUM(45),(cl_objectfn)L20make_inet_address,Cnil,MAKE_FIXNUM(17376)},
{0,0,2,0,MAKE_FIXNUM(193),MAKE_FIXNUM(48),(cl_objectfn)L22make_inet_socket,Cnil,MAKE_FIXNUM(17843)},
{0,0,-1,0,MAKE_FIXNUM(205),MAKE_FIXNUM(61),(cl_objectfn)L48make_stream_from_fd,Cnil,MAKE_FIXNUM(37189)},
{0,0,2,0,MAKE_FIXNUM(209),MAKE_FIXNUM(72),(cl_objectfn)LC51define_socket_condition,Cnil,MAKE_FIXNUM(40473)},
{0,0,1,0,MAKE_FIXNUM(210),MAKE_FIXNUM(100),(cl_objectfn)L52condition_for_errno,Cnil,MAKE_FIXNUM(42350)},
{0,0,1,0,MAKE_FIXNUM(211),MAKE_FIXNUM(103),(cl_objectfn)L53socket_error,Cnil,MAKE_FIXNUM(42460)},
{0,0,1,0,MAKE_FIXNUM(212),MAKE_FIXNUM(105),(cl_objectfn)L54name_service_error,Cnil,MAKE_FIXNUM(42828)},
{0,0,2,0,MAKE_FIXNUM(215),MAKE_FIXNUM(109),(cl_objectfn)LC56define_name_service_condition,Cnil,MAKE_FIXNUM(43870)},
{0,0,1,0,MAKE_FIXNUM(216),MAKE_FIXNUM(123),(cl_objectfn)L57condition_for_name_service_errno,Cnil,MAKE_FIXNUM(44767)},
{0,0,0,0,MAKE_FIXNUM(217),MAKE_FIXNUM(124),(cl_objectfn)L58get_name_service_errno,Cnil,MAKE_FIXNUM(44908)},
{0,0,1,0,MAKE_FIXNUM(218),MAKE_FIXNUM(125),(cl_objectfn)L59get_name_service_error_message,Cnil,MAKE_FIXNUM(45034)},
{0,0,2,0,MAKE_FIXNUM(219),MAKE_FIXNUM(126),(cl_objectfn)L60get_sockopt_int,Cnil,MAKE_FIXNUM(45382)},
{0,0,2,0,MAKE_FIXNUM(220),MAKE_FIXNUM(127),(cl_objectfn)L61get_sockopt_bool,Cnil,MAKE_FIXNUM(45823)},
{0,0,2,0,MAKE_FIXNUM(221),MAKE_FIXNUM(128),(cl_objectfn)L62get_sockopt_timeval,Cnil,MAKE_FIXNUM(46337)},
{0,0,3,0,MAKE_FIXNUM(222),MAKE_FIXNUM(129),(cl_objectfn)L63set_sockopt_int,Cnil,MAKE_FIXNUM(46862)},
{0,0,3,0,MAKE_FIXNUM(223),MAKE_FIXNUM(130),(cl_objectfn)L64set_sockopt_bool,Cnil,MAKE_FIXNUM(47279)},
{0,0,3,0,MAKE_FIXNUM(224),MAKE_FIXNUM(131),(cl_objectfn)L65set_sockopt_timeval,Cnil,MAKE_FIXNUM(47716)},
{0,0,2,0,MAKE_FIXNUM(225),MAKE_FIXNUM(136),(cl_objectfn)LC66define_sockopt,Cnil,MAKE_FIXNUM(48336)},
{0,0,1,0,MAKE_FIXNUM(226),MAKE_FIXNUM(138),(cl_objectfn)L67sockopt_type,Cnil,MAKE_FIXNUM(48830)},
{0,0,1,0,MAKE_FIXNUM(227),MAKE_FIXNUM(140),(cl_objectfn)L68sockopt_receive_buffer,Cnil,MAKE_FIXNUM(48877)},
{0,0,2,0,MAKE_FIXNUM(228),MAKE_FIXNUM(141),(cl_objectfn)L69_setf_sockopt_receive_buffer_,Cnil,MAKE_FIXNUM(48877)},
{0,0,1,0,MAKE_FIXNUM(229),MAKE_FIXNUM(143),(cl_objectfn)L70sockopt_receive_timeout,Cnil,MAKE_FIXNUM(48933)},
{0,0,2,0,MAKE_FIXNUM(230),MAKE_FIXNUM(144),(cl_objectfn)L71_setf_sockopt_receive_timeout_,Cnil,MAKE_FIXNUM(48933)},
{0,0,1,0,MAKE_FIXNUM(231),MAKE_FIXNUM(146),(cl_objectfn)L72sockopt_send_timeout,Cnil,MAKE_FIXNUM(48996)},
{0,0,2,0,MAKE_FIXNUM(232),MAKE_FIXNUM(147),(cl_objectfn)L73_setf_sockopt_send_timeout_,Cnil,MAKE_FIXNUM(48996)},
{0,0,1,0,MAKE_FIXNUM(233),MAKE_FIXNUM(149),(cl_objectfn)L74sockopt_reuse_address,Cnil,MAKE_FIXNUM(49056)},
{0,0,2,0,MAKE_FIXNUM(234),MAKE_FIXNUM(150),(cl_objectfn)L75_setf_sockopt_reuse_address_,Cnil,MAKE_FIXNUM(49056)},
{0,0,1,0,MAKE_FIXNUM(235),MAKE_FIXNUM(152),(cl_objectfn)L76sockopt_keep_alive,Cnil,MAKE_FIXNUM(49115)},
{0,0,2,0,MAKE_FIXNUM(236),MAKE_FIXNUM(153),(cl_objectfn)L77_setf_sockopt_keep_alive_,Cnil,MAKE_FIXNUM(49115)},
{0,0,1,0,MAKE_FIXNUM(237),MAKE_FIXNUM(155),(cl_objectfn)L78socket_dont_route,Cnil,MAKE_FIXNUM(49171)},
{0,0,2,0,MAKE_FIXNUM(238),MAKE_FIXNUM(156),(cl_objectfn)L79_setf_socket_dont_route_,Cnil,MAKE_FIXNUM(49171)},
{0,0,1,0,MAKE_FIXNUM(239),MAKE_FIXNUM(158),(cl_objectfn)L80socket_linger,Cnil,MAKE_FIXNUM(49226)},
{0,0,2,0,MAKE_FIXNUM(240),MAKE_FIXNUM(159),(cl_objectfn)L81_setf_socket_linger_,Cnil,MAKE_FIXNUM(49226)},
{0,0,1,0,MAKE_FIXNUM(241),MAKE_FIXNUM(161),(cl_objectfn)L82sockopt_tcp_nodelay,Cnil,MAKE_FIXNUM(49370)},
{0,0,2,0,MAKE_FIXNUM(242),MAKE_FIXNUM(162),(cl_objectfn)L83_setf_sockopt_tcp_nodelay_,Cnil,MAKE_FIXNUM(49370)},
};
